==> liquidsoap_0.9.1.orig.tar.gz: liquidsoap-0.9.1/src/operators/ms_stereo.ml <==
(*****************************************************************************

  Liquidsoap, a programmable audio stream generator.
  Copyright 2003-2009 Savonet team

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details, fully stated in the COPYING
  file at the root of the liquidsoap distribution.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *****************************************************************************)

open Source

type mode = Encode | Decode

class msstereo (source:source) mode width =
object (self)
  inherit operator [source] as super

  method stype = source#stype
  method is_ready = source#is_ready
  method remaining = source#remaining
  method abort_track = source#abort_track

  method get_frame buf =
    let offset = AFrame.position buf in
      source#get buf ;
      let buffer = AFrame.get_float_pcm buf in
        for i = offset to AFrame.position buf -1 do
          match mode with
            | Encode ->
              let left = buffer.(0).(i) and right = buffer.(1).(i) in
                buffer.(0).(i) <- 0.5 *. (left +. right) ; (* mid *)
                buffer.(1).(i) <- 0.5 *. (left -. right)   (* side *)
            | Decode ->
              let mid = buffer.(0).(i) and side = buffer.(1).(i) in
                buffer.(0).(i) <- mid +. side *. width ; (* left *)
                buffer.(1).(i) <- mid -. side *. width   (* right *)
        done
end

let () =
  Lang.add_operator "stereo.ms.encode"
    [ "", Lang.source_t, None, None ]
    ~category:Lang.SoundProcessing
    ~descr:"Encode left+right stereo to mid+side stereo (M/S)."
    (fun p ->
       let s = Lang.to_source (Lang.assoc "" 1 p) in
         ((new msstereo s Encode 0.):>source))

let () =
  Lang.add_operator "stereo.ms.decode"
    [ "width", Lang.float_t,  Some (Lang.float 1.),
      Some "Width of the stereo field." ;
      "", Lang.source_t, None, None ]
    ~category:Lang.SoundProcessing
    ~descr:"Decode mid+side stereo (M/S) to left+right stereo."
    (fun p ->
       let s = Lang.to_source (Lang.assoc "" 1 p) in
       let w = Lang.to_float (Lang.assoc "width" 1 p) in
         ((new msstereo s Decode w):>source))

==> liquidsoap_0.9.2.orig.tar.gz: liquidsoap-0.9.2/src/operators/ms_stereo.ml <==
(*****************************************************************************

  Liquidsoap, a programmable audio stream generator.
  Copyright 2003-2009 Savonet team

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details, fully stated in the COPYING
  file at the root of the liquidsoap distribution.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *****************************************************************************)

open Source

type mode = Encode | Decode

class msstereo (source:source) mode width =
object (self)
  inherit operator [source] as super

  method stype = source#stype
  method is_ready = source#is_ready
  method remaining = source#remaining
  method abort_track = source#abort_track

  method private get_frame buf =
    let offset = AFrame.position buf in
      source#get buf ;
      let buffer = AFrame.get_float_pcm buf in
        for i = offset to AFrame.position buf -1 do
          match mode with
            | Encode ->
              let left = buffer.(0).(i) and right = buffer.(1).(i) in
                buffer.(0).(i) <- 0.5 *. (left +. right) ; (* mid *)
                buffer.(1).(i) <- 0.5 *. (left -. right)   (* side *)
            | Decode ->
              let mid = buffer.(0).(i) and side = buffer.(1).(i) in
                buffer.(0).(i) <- mid +. side *. width ; (* left *)
                buffer.(1).(i) <- mid -. side *. width   (* right *)
        done
end

let () =
  Lang.add_operator "stereo.ms.encode"
    [ "", Lang.source_t, None, None ]
    ~category:Lang.SoundProcessing
    ~descr:"Encode left+right stereo to mid+side stereo (M/S)."
    (fun p _ ->
       let s = Lang.to_source (Lang.assoc "" 1 p) in
         new msstereo s Encode 0.)

let () =
  Lang.add_operator "stereo.ms.decode"
    [ "width", Lang.float_t,  Some (Lang.float 1.),
      Some "Width of the stereo field." ;
      "", Lang.source_t, None, None ]
    ~category:Lang.SoundProcessing
    ~descr:"Decode mid+side stereo (M/S) to left+right stereo."
    (fun p _ ->
       let s = Lang.to_source (Lang.assoc "" 1 p) in
       let w = Lang.to_float (Lang.assoc "width" 1 p) in
         new msstereo s Decode w)
